var SourceMapGenerator, cache, commentSymbols, deindent, emptyRE, hash, parse5, splitRE, validateTemplate;

parse5 = require('parse5');

validateTemplate = require('vue-template-validator');

cache = require('lru-cache')(100);

SourceMapGenerator = require('source-map').SourceMapGenerator;

hash = require('hash-sum');

deindent = require('de-indent');

splitRE = /\r?\n/g;

emptyRE = /^\s*$/;

commentSymbols = {
  'iced': '#',
  'iced-jsx': '#',
  'iced-redux': '#',
  'coffee': '#',
  'coffee-jsx': '#',
  'coffee-redux': '#',
  'purs': '--',
  'ulmus': '--'
};

function commentScript(content, lang) {
  var lines, symbol;
  symbol = getCommentSymbol(lang);
  lines = content.split(splitRE);
  return lines.map(function(line, index) {
    if (index === lines.length - 1 && emptyRE.test(line)) {
      return '';
    } else {
      return symbol + (emptyRE.test(line) ? '' : ' ' + line);
    }
  }).join('\n');
};

function getCommentSymbol(lang) {
  return commentSymbols[lang] || '//';
};

function getAttribute(node, name) {
  var attr, i;
  if (node.attrs) {
    i = node.attrs.length;
    attr = void 0;
    while (i--) {
      attr = node.attrs[i];
      if (attr.name === name) {
        return attr.value;
      }
    }
  }
};

module.exports = function(content, filename, needMap) {
  var cacheKey, filenameWithHash, fragment, output;
  cacheKey = hash(filename + content);
  filenameWithHash = filename + '?' + cacheKey;
  output = cache.get(cacheKey);
  if (output) {
    return output;
  }
  output = {
    template: [],
    style: [],
    script: [],
    styleImports: []
  };
  fragment = parse5.parseFragment(content, {
    locationInfo: true
  });
  fragment.childNodes.forEach(function(node) {
    var end, isCommentLine, lang, lineOffset, map, result, scoped, src, start, type, warnings;
    type = node.tagName;
    lang = getAttribute(node, 'lang');
    src = getAttribute(node, 'src');
    scoped = !!getAttribute(node, 'scoped');
    warnings = null;
    map = null;
    if (!output[type]) {
      return;
    }
    if ((type === 'script' || type === 'template') && output[type].length > 0) {
      throw new Error('[vue-loader] Only one <script> or <template> tag is ' + 'allowed inside a Vue component.');
    }
    if (src) {
      if (type === 'style') {
        output.styleImports.push({
          src: src,
          lang: lang,
          scoped: scoped
        });
      } else if (type === 'template') {
        output.template.push({
          src: src,
          lang: lang
        });
      } else if (type === 'script') {
        output.script.push({
          src: src,
          lang: lang
        });
      }
      return false;
    }
    if (type !== 'template' && (!node.childNodes || !node.childNodes.length)) {
      return;
    }
    if (type === 'template') {
      node = node.content;
      if (!lang) {
        warnings = validateTemplate(node, content);
      }
    }
    start = node.childNodes[0].__location.startOffset;
    end = node.childNodes[node.childNodes.length - 1].__location.endOffset;
    result = void 0;
    if (type === 'script') {
      result = commentScript(content.slice(0, start), lang) + deindent(content.slice(start, end)) + commentScript(content.slice(end), lang);
    } else {
      lineOffset = content.slice(0, start).split(splitRE).length - 1;
      result = deindent(content.slice(start, end));
      result = Array(lineOffset + 1).join('\n') + result;
    }
    if (needMap) {
      map = new SourceMapGenerator;
      map.setSourceContent(filenameWithHash, content);
      isCommentLine = function(line) {
        return type === 'script' && line.indexOf(getCommentSymbol(lang)) === 0;
      };
      result.split(splitRE).forEach(function(line, index) {
        if (!emptyRE.test(line) && !isCommentLine(line)) {
          return map.addMapping({
            source: filenameWithHash,
            original: {
              line: index + 1,
              column: 0
            },
            generated: {
              line: index + 1,
              column: 0
            }
          });
        }
      });
      if (type === 'script' && !lang) {
        result += '\n/* generated by vue-loader */\n';
      }
    }
    return output[type].push({
      lang: lang,
      scoped: scoped,
      content: result,
      map: map && map.toJSON(),
      warnings: warnings
    });
  });
  cache.set(cacheKey, output);
  return output;
};
